//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#include "SearchListCtrl.h"
#include "afxcmn.h"
#include "packets.h"
#include "types.h"
#include "afxwin.h"
#include "IconStatic.h"
#include "ClosableTabCtrl.h"
#include "Loggable.h"

class CCustomAutoComplete;

typedef struct
{
	CString sNotSearch;
	bool bDocumentSearch;
} SearchData;
typedef CMap<int, int, SearchData, SearchData> SearchMap;

// CSearchDlg dialog
class CSearchDlg : public CResizableDialog, public CLoggable
{
	DECLARE_DYNAMIC(CSearchDlg)

public:
	CSearchDlg(CWnd* pParent = NULL);   // standard constructor
	virtual ~CSearchDlg();

	afx_msg void OnBnClickedStarts();
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnBnClickedCancels();
	afx_msg void OnBnClickedSdownload();
	afx_msg void OnBnClickedClearall();
	afx_msg void OnChangeMax();
	afx_msg void OnChangeMin();
	afx_msg void OnSearchKeyDown();
	afx_msg void OnBnClickedMores();

	enum { IDD = IDD_SEARCH };

	void	Localize();
	void	DownloadSelected(bool bPaused = false);
	void	LocalSearchEnd(uint16 count, bool LocalSearchEnd);
	void	AddUDPResult(uint16 count);
	void	DeleteSearch(uint32 nSearchID);
	void	DeleteAllSearchs();
	void	CreateNewTab(const CString &strSearchString, uint32 nSearchID);
	void	UpdateCatTabs();
	void	OnChangeSize(bool changedMin);
	void	SearchClipBoard();
	void	IgnoreClipBoardLinks(CString strLink)		{m_strLastClipBoard = strLink; }
	void	AddEd2kLinksToDownload(CString strLink,EnumCategories eCatID);
	void	AddEd2kLinksToDownload(CString strLink)		{AddEd2kLinksToDownload(strLink,CCat::GetCatIDByIndex(m_ctrlCatTabs.GetCurSel()));}
	CSearchListCtrl m_ctlSearchList;
	CClosableTabCtrl m_ctlSearchTabs;
	void	DoNewSearch(CString strSearch, CString strTypeText, unsigned long dwMinSize, unsigned long dwMaxSize,
							 int iAvailability, CString strExtension, bool bDoGlobal, CString strSearchExclusion, uint16 nSearchID = 0);
	CString	GetNotSearch(int nSearchId);
	bool	IsDocumentSearch(int nSearchId);
	BOOL	SaveSearchStrings();
	void	ShowSearchTabs(int iCmdShow);
	bool	IsLastSearchCanceled() const {return m_bCancelled;}
	bool	IsMoreEnabled() const {return m_bMoreEnabled;}
	CPPToolTip m_ttip;

protected:
	void StartNewSearch();
	LRESULT OnCloseTab(WPARAM wparam, LPARAM lparam);
	LRESULT OnShowWindow(WPARAM wparam, LPARAM lparam);
	CString	CreateWebQuery();

	virtual void DoDataExchange(CDataExchange* pDX);
	virtual BOOL OnInitDialog();
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	afx_msg void OnNMDblclkSearchlist(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnTcnSelchangeTab1(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnEnChangeSearchname();
	afx_msg void OnEnChangeMethod();
	afx_msg BOOL OnToolTipNotify(UINT id, NMHDR *pNMH, LRESULT *pResult);
	afx_msg void OnBnClickedSearchReset();
	afx_msg void OnBnClickedDownloadAllEd2k();
	afx_msg void OnDestroy();

	DECLARE_MESSAGE_MAP()

private:
	Packet*		m_pGlobalSearchPacket;
	UINT_PTR	global_search_timer;
	CProgressCtrl m_ctlGlobalSearchProgress;
	bool		m_bCancelled;
	uint16		m_uServerCount;
	bool		m_bGlobalSearch;
	bool		m_bMoreEnabled;
	uint32		m_nSearchID;
	CComboBoxEx	methodbox;
	CComboBox	Stypebox;
	CTabCtrl	m_ctrlCatTabs;
	CString		m_strLastClipBoard;
	bool		m_bGuardCBPrompt;
	CImageList	m_BoxImageList;
	CIconStatic	m_ctrlSearchFrm;
	CIconStatic	m_ctrlDirectDlFrm;
	CCustomAutoComplete* m_pacSearchString;

	HICON oldSearchLstIcon;
	SearchMap m_SearchMap;

	uint32	m_nMoreRequestCount;

//	Constants definition
	static const long SEARCH_PARAMID_TYPE = 0x00030001;
	static const long SEARCH_PARAMID_EXTENSION = 0x00040001;
	static const long SEARCH_PARAMID_AVAILABILITY = 0x15000101;
	static const long SEARCH_PARAMID_MINSIZE = 0x02000101;
	static const long SEARCH_PARAMID_MAXSIZE = 0x02000102;

	static const byte SEARCH_PARAMTYPE_STRING = 1;
	static const byte SEARCH_PARAMTYPE_TYPE = 2;
	static const byte SEARCH_PARAMTYPE_NUMERIC = 3;

	static const uint16 SEARCH_OPPARAM_AND = 0x0000;
	static const uint16 SEARCH_OPPARAM_OR  = 0x0100;
	static const uint16 SEARCH_OPPARAM_NOT = 0x0200;

private:
	CString		ToQueryString(const CString &str);
	CStatic*	GetSearchLstStatic() const		{ return (CStatic*)GetDlgItem(IDC_SEARCHLST_ICO); }
	HICON		GetSearchLstIcon() const		{ return (HICON)::LoadImage(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_SEARCHRESULTS), IMAGE_ICON, 16, 16, 0); }
};
