#include "stdafx.h"
#include "SourceSaver.h"
#include "PartFile.h"
#ifndef NEW_SOCKETS_ENGINE
#include "emule.h"
#else
#include "otherfunctions.h"
#endif //NEW_SOCKETS_ENGINE

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef NEW_SOCKETS_ENGINE
CSourceSaver::CSourceData::CSourceData(CUpDownClient *pClient, const char *exp)
{
	sourceID = pClient->GetUserID();
	sourcePort = pClient->GetUserPort();
	partsavailable = pClient->GetAvailablePartCount();
	memcpy(expiration, exp, 6);
	expiration[6] = 0;
}
#endif //NEW_SOCKETS_ENGINE
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSourceSaver::CSourceData::CSourceData(uint32 dwID, uint16 uPort, const char *exp)
{
	sourceID = dwID;
	sourcePort = uPort;
	memcpy(expiration, exp, 6);
	expiration[6] = 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSourceSaver::CSourceData::CSourceData(CSourceData *pOld)
{
	sourceID = pOld->sourceID; 
	sourcePort = pOld->sourcePort; 
	memcpy(expiration, pOld->expiration, 6); 
	partsavailable = pOld->partsavailable;
	expiration[6] = 0;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSourceSaver::CSourceSaver(void)
{
	m_dwLastTimeSaved = ::GetTickCount() - rand() / 2;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CSourceSaver::~CSourceSaver(void)
{
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Process() saves/loads up to 'maxSourcesToSave' sources for 'file' if the auto save/load timers
//		have expired. It returns false if the sources were not saved.
bool CSourceSaver::Process(CPartFile *file, int maxSourcesToSave, bool bIgnoreTimer /* = false */)
{
	uint32	dwTicks = ::GetTickCount();

	//	If the auto-save time (fixed 30 minutes) has passed...
	if (((dwTicks - m_dwLastTimeSaved) > RESAVETIME) || bIgnoreTimer)
	{
		//	Set next auto-save time + random 0..16 seconds
		m_dwLastTimeSaved = dwTicks - rand() / 2;

		SourceList srcs;

		//	Source file name is .met file name .txtsrc in temp directory
		CString strSrcFile = RemoveFileExtension(file->GetFullName());
		
		strSrcFile += _T(".txtsrc");
		LoadSourcesFromFile(file, &srcs, strSrcFile);
		SaveSources(file, &srcs, strSrcFile, maxSourcesToSave);

		//	Delete the old sources list
		while (!srcs.IsEmpty())
			delete srcs.RemoveHead();

		return true;
	}
	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSourceSaver::LoadSources(CPartFile* file)
{
	SourceList srcs;

	//	Source file name is .met file name .txtsrc in temp directory
	CString strSrcFile = RemoveFileExtension(file->GetFullName());
	
	strSrcFile += _T(".txtsrc");
	LoadSourcesFromFile(file, &srcs, strSrcFile);

	AddSourcesToDownload(file, &srcs);

	while (!srcs.IsEmpty())
		delete srcs.RemoveHead();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	LoadSourcesFromFile() loads the sources for part file 'file' from the source file 'slsfile'
//		and adds them to the source list 'sources'.
void CSourceSaver::LoadSourcesFromFile(CPartFile* file, SourceList* sources, const CString& slsfile)
{
	CStdioFile f;

//	Open the source file. If it can't be opened, just return.
	if (!f.Open(slsfile, CFile::modeRead | CFile::shareDenyWrite | CFile::typeText))
		return;

	CString strLine, strTmp;

	//	Until we hit the EOF...
	while(f.ReadString(strLine))
	{
	//	If it's a comment line, skip it
		if (strLine.GetAt(0) == '#')
			continue;

		int pos = strLine.Find(':');

	//	If the line doesn't contain the ':' for an ip:port pair, skip it
		if (pos < 0)
			continue;

	//	Strip off the IP and convert it to a 32bit integer
		strTmp = strLine.Left(pos);
		strLine = strLine.Mid(pos+1);

		uint32 dwID = inet_addr(strTmp);

	//	If the IP is invalid, skip the line
		if (dwID == INADDR_NONE)
			continue;

	//	If the line doesn't contain the ',' seperating the date, skip it
		pos = strLine.Find(',');
		if (pos < 0)
			continue;

	//	Strip off the port and convert it to an integer
		strTmp = strLine.Left(pos);
		strLine = strLine.Mid(pos+1);

		uint16 wPort = atoi(strTmp);

	//	If the port is invalid, skip the line
		if (!wPort)
			continue;

	//	If the line doesn't contain the terminating ';', skip it
		pos = strLine.Find(';');
		if (pos < 0)
			continue;

	//	Strip off the expiration date
		strTmp = strLine.Left(pos);
		if (IsExpired(strTmp))
			continue;

	//	If we got a valid, unexpired source, add it to the 'sources' list
		CSourceData* newsource = new CSourceData(dwID, wPort, strTmp);

		sources->AddTail(newsource);
	}
	f.Close();
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CSourceSaver::AddSourcesToDownload(CPartFile* file, SourceList* sources)
{
#ifndef NEW_SOCKETS_ENGINE
	//	For each SourceData...
	for (POSITION pos = sources->GetHeadPosition(); pos; sources->GetNext(pos))
	{
	//	If we did not reach  the maximum number of sources, allow to add some more
		if (file->GetSourceCount() < g_eMuleApp.m_pGlobPrefs->GetMaxSourcePerFile())
		{
			CSourceData* cur_src = sources->GetAt(pos);
			//CUpDownClient* newclient = new CUpDownClient(cur_src->sourcePort, cur_src->sourceID, 0, 0, file);
			//g_eMuleApp.m_pDownloadQueue->CheckAndAddSource(file, newclient);

		 	CUpDownClient* pNewSource = g_eMuleApp.m_pDownloadQueue->CheckAndAddSource(file, cur_src->sourceID, cur_src->sourcePort, 0, 0);
			if (pNewSource)
				pNewSource->SetUserName(GetResString(IDS_SAVED_SOURCE));
		}
	}

	if (sources->GetCount() > 0)
		g_eMuleApp.m_pdlgEmule->AddDebugLogLine(_T("Loaded %i sources for file %s"), sources->GetCount(), file->GetFileName());
#endif //NEW_SOCKETS_ENGINE
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	SaveSources() writes to file 'slsfile' a list of up to 'maxSourcesToSave' sources composed of the
//		currently available sources with the highest number of available parts and the previously
//		saved sources in 'prevsources'.
void CSourceSaver::SaveSources(CPartFile* file, SourceList* prevsources, const CString& slsfile, int maxSourcesToSave)
{
#ifndef NEW_SOCKETS_ENGINE
	SourceList srcstosave;
	CSourceData* sourcedata;

	ASSERT(srcstosave.IsEmpty());

	POSITION pos, pos2;

	//
	// Choose best sources for the file
	//

	//	For each source slot...
	for (int sl=0;sl<SOURCESSLOTS;sl++)
	{
		//	If the slot contains sources...
		if (!file->m_srcLists[sl].IsEmpty())
		{
			//	For each source in the slot...
			for (pos = file->m_srcLists[sl].GetHeadPosition(); pos != NULL; file->m_srcLists[sl].GetNext(pos))
			{
				CUpDownClient* cur_src = file->m_srcLists[sl].GetAt(pos);

			//	If it's a LOWID source, skip it
				if (cur_src->HasLowID())
					continue;

				if (srcstosave.IsEmpty())
				{
					sourcedata = new CSourceData(cur_src, CalcExpiration(EXPIREIN));
					srcstosave.AddHead(sourcedata);
					continue;
				}
				//	If the list isn't full or the source has more parts than the lowest in the list...
				if ((srcstosave.GetCount() < maxSourcesToSave) || (cur_src->GetAvailablePartCount() > srcstosave.GetTail()->partsavailable))
				{
				//	If the list is full, remove the source with the lowest number of parts
					if (srcstosave.GetCount() == maxSourcesToSave)
						delete srcstosave.RemoveTail();

					ASSERT(srcstosave.GetCount() < maxSourcesToSave);

				//
				//	Add the new source to the list in "# avail parts" order
				//
					bool bInserted = false;

				//	For each SourceData, (backwards starting with the last)...
					for (pos2 = srcstosave.GetTailPosition();pos2 != 0;srcstosave.GetPrev(pos2))
					{
						CSourceData* cur_srctosave = srcstosave.GetAt(pos2);

						if (cur_srctosave->partsavailable > cur_src->GetAvailablePartCount())
						{
							sourcedata = new CSourceData(cur_src, CalcExpiration(EXPIREIN));
							srcstosave.InsertAfter(pos2, sourcedata);
							bInserted = true;
							break;
						}
					}
					if (!bInserted)
					{
						sourcedata = new CSourceData(cur_src, CalcExpiration(EXPIREIN));
						srcstosave.AddHead(sourcedata);
					}
				}
			}
		}
	}

	// Add previously saved sources if found sources does not reach the limit
	for (pos = prevsources->GetHeadPosition(); pos; prevsources->GetNext(pos))
	{
		CSourceData* cur_sourcedata = prevsources->GetAt(pos);

		if (srcstosave.GetCount() == maxSourcesToSave)
			break;

		ASSERT(srcstosave.GetCount() <= maxSourcesToSave);

		bool bFound = false;

		for (pos2 = srcstosave.GetHeadPosition(); pos2; srcstosave.GetNext(pos2))
		{
			if (srcstosave.GetAt(pos2)->Compare(cur_sourcedata))
			{
				bFound = true;
				break;
			}
		}
		if (!bFound)
		{
			srcstosave.AddTail(new CSourceData(cur_sourcedata));
		}
	}

//	g_eMuleApp.m_pdlgEmule->AddDebugLogLine(_T("Saving %i sources for file %s to slsfile"), srcstosave.GetCount(), file->GetFileName());

	CString strLine;
	CStdioFile f;

	if (!f.Open(slsfile, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite | CFile::typeText))
		return;
	f.WriteString("#link: " + file->CreateED2KSourceLink(2,10, true) + "\n\n");
	f.WriteString("#format: a.b.c.d:port,expirationdate(yymmdd);\n");
	while (!srcstosave.IsEmpty())
	{
		CSourceData* cur_src = srcstosave.RemoveHead();
		uint32 dwID = cur_src->sourceID;
		uint16 wPort = cur_src->sourcePort;

		strLine.Format("%i.%i.%i.%i:%i,%s;\n", (byte)dwID,(byte)(dwID>>8),(byte)(dwID>>16),(byte)(dwID>>24), wPort, cur_src->expiration);
		delete cur_src;
		f.WriteString(strLine);
	}
	f.Close();
#endif //NEW_SOCKETS_ENGINE
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	CalcExpiration() calculates a date 'nDays' from the current date and returns it as a string in
//		'yymmdd' format.
CString CSourceSaver::CalcExpiration(int nDays)
{
	CTime expiration = CTime::GetCurrentTime();
	CTimeSpan timediff(nDays, 0, 0, 0);

	expiration += timediff;

	CString strExpiration;

	strExpiration.Format("%02i%02i%02i", (expiration.GetYear() % 100), expiration.GetMonth(), expiration.GetDay());

	return strExpiration;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	IsExpired() returns true if 'expirationdate' is less than the current date or if it has an invalid format.
bool CSourceSaver::IsExpired(const CString &strExpirationDate)
{
	//	Format check. CSourceData code naively relies on correct format
	if (strExpirationDate.GetLength() == 6)
	{
		int		year = atoi(strExpirationDate.Mid(0, 2)) + 2000;
		int		month = atoi(strExpirationDate.Mid(2, 2));
		int		day = atoi(strExpirationDate.Mid(4, 2));

		CTime	expiration(year, month, day, 0, 0, 0);

		return (expiration < CTime::GetCurrentTime());
	}
	else
		return true;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
